// mozileCopyCutPaste.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Copy Cut Paste Javascript
Mozile version 0.6.16

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-11-16

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	This file handles the mozileCopu, mozileCut, and mozilePaste functions,
as well as some other code related to the clipboard.

*********/



/********** Clipboard Functionality *************/
/*********Try to cleanout some real bad HTML to make it parseable***********/	
function mozileCleanUpHTML(dirtyHTML) {
	var f = ["mozileCopyCutPaste.js","mozileCleanUpHTML"];
	mozileDebug(f,2,"Cleaning HTML...");
	
	try{
		//Basic html4 cleanup:
		re = /<br[^>]*>/gi
		dirtyHTML	=	dirtyHTML.replace(re,"<br />")
		
		re = /<img +[^>]+>/gi									//find <img.......>
		dirtyHTML	=	dirtyHTML.replace(re,"")			//remove <img> tags
		
		re = /&/gi													//find &
		dirtyHTML	=	dirtyHTML.replace(re,"&amp;")	//replace with &amp;
		
		re=/<spacer\w{0,}[^>]*(>)/gi							//custom <spacer tag - CLOSE IT!
		dirtyHTML	=	dirtyHTML.replace(re,"/>")
		
		re=/(<script .*|\s<\/script>)/gi
		dirtyHTML	=	dirtyHTML.replace(re,"")			//remove <script> tags
		
		re=/(<!--.*-->)/gi
		dirtyHTML	=	dirtyHTML.replace(re,"")			//remove comments tags
		
		return dirtyHTML;
		
	}	catch(e) {
			mozileDebug(f,1,"An error occured during HTML clean up: "+e);
			return false;
	}
}
	
	
/*********Convert string to a document fragment*************/	
function mozileStrToDOM(xmlStr, namespace, wrapperNodeName) {
	var f = ["mozileCopyCutPaste.js","mozileStrToDOM"];
	mozileDebug(f,2,"Converting string to DOM fragment...");
	
	if(!namespace || typeof(namespace)=="undefined" || namespace=="")
		namespace	=	"http://www.w3.org/1999/xhtml"								//Default namespace is xhtml
		
	if(!wrapperNodeName || typeof(wrapperNodeName)=="undefined" || wrapperNodeName=="")
		wrapperNodeName	=	"removeme"												//Default nodetype to use to wrap up string to create document fragment
	
	try{	
		p	=	new DOMParser()																//Instantiate DOMParser
		try{
			str = "<"+wrapperNodeName+" xmlns=\""+namespace+"\">"+xmlStr+"</"+wrapperNodeName+">"
			docFrag = p.parseFromString(str,"text/xml")
			
			//Need to insert code that look for parser errors here. Copy that stuff from usenet.
			mozileDebug(f,1,xmlStr +" === "+ docFrag +" ==== "+ printXML(docFrag));
			return docFrag
		}	catch(e) {
				mozileDebug(f,1,"mozileStrToDOM():: Impossible to parse string: "+str+" Error:"+e);
				return false;
		}
	}	catch(e) {
		mozileDebug(f,1,"mozileStrToDOM():: Could not create DOMParser object. Error: "+e);
		return false;
	}
}

// Mozile Paste
// Takes the content of the system clipboard and inserts it at the current selection.
// Right now only plain text is handled.

function mozilePaste() {
	var f = ["mozileCopyCutPaste.js","mozilePaste"];
	mozileDebug(f,2,"Pasting");
	
	//I inserted this code to make things work in my setup, dunno if it is "bad" or acceptable for all... (doesn't do much if mozileSelection is ok, though...)
	if(!mozileSelection) {
		if(!mozileDetect()) {
			return false;	
		}
	}
	


/*
	var preferredFlavor;
	try {
		preferredFlavor = mozileTarget.document.MOZILE_RSD_CONFIG["pasteMode"];
	} catch(e) {
		preferredFlavor = "text";
	}
*/

		// Until fixed, use Text only.
	var preferredFlavor = "text"
	
	
	mozileDebug(f,2,"Preferred Flavor:"+preferredFlavor);

	var clipboard = mozilla.getClipboard();
	// only get text for now: POST04: need XHTML validator first so that correct
	// HTML comes first.

/*
	if(preferredFlavor=="HTML" || preferredFlavor=="XML") {
		try {
			html	=	clipboard.getData(MozClipboard.HTML_FLAVOR)		//Using the XML_FLAVOR really does not make much sense as it has to be set to "text/html" anyway (otherwise the getData only grabs text)
			//html	=	mozileCleanUpHTML(html)		//Should this be optional? I think not, 'cause of security concerns... Still...
			mozileDebug(f,2,"HTML: "+html);
			//var xdoc = document.createDocumentFragment();
			//xdoc.appendChild(document.createTextNode(html));
			xdoc	=	mozileStrToDOM(html);
			mozileDebug(f,2,"DOM: "+xdoc);
			
			
			// insertNode is badly broken, so we have to do things ourselves.
			
			//mozileDebug(f,1,"Offset: "+ mozileSelection.anchorNode + mozileSelection.anchorOffset);
			// split the current text node
			mozileSelection.anchorNode.splitText(mozileSelection.anchorOffset);
			
			// insert the document element 			
			var parent = mozileSelection.anchorNode.parentNode;
			var next = mozileSelection.anchorNode.nextSibling;
			parent.insertBefore(xdoc.documentElement,next);
			
			
//			var children = xdoc.documentElement.childNodes;
//			mozileDebug(f,1,"Length: "+ children.length + xdoc.documentElement.nodeName + xdoc.documentElement.childNodes.length);
//			for(var i=0;i<children.length;i++) {
//				parent.insertBefore(children[i], next);
//			}
			mozileDebug(f,2,"Node inserted?");
			return;
			
		} catch(e) {
			mozileDebug(f,1,"Could not get data from clipboard (HTML/XML mode). Error: "+e);
			preferredFlavor = "text";
		}
		
	}
*/
	
	//Default to text/unicode
	if(preferredFlavor=="text" || typeof(preferredFlavor)=="undefined" || preferredFlavor=="") {		
		var text = clipboard.getData(MozClipboard.TEXT_FLAVOR);
		if(text) {
			mozileSelection.insertNode(mozileTarget.document.createTextNode(text));
		}
	}

	mozileDo("Paste");		//Shouldn't this be at the end of the function? If an error occurs and the paste does not occur, we'd have an extra item on the undo stack...
	return true;
	
} // end mozilePaste()


/*
// Mozile Paste
// Takes the content of the system clipboard and inserts it at the current selection.
// Right now only plain text is handled.

function mozilePaste() {
	mozileDo("Paste");

	var clipboard = mozilla.getClipboard();
	// only get text for now: POST04: need XHTML validator first so that correct
	// HTML comes first.
	var text = clipboard.getData(MozClipboard.TEXT_FLAVOR);
	if(text)
	{
		mozileSelection.insertNode(mozileTarget.document.createTextNode(text));
	}
} // end mozilePaste()
*/


// Mozile Copy
// Copies the selected text to the system clipboard.
// The selection is currently copied as plain text.

function mozileCopy() {
	mozileDo("Copy");

	var cssr = mozileSelection.getEditableRange();

	if(!cssr || cssr.collapsed) // not an editable area or nothing selected
		return false;

	// data to save - render as text (temporary thing - move to html later)
	var text = cssr.toString();

	var clipboard = mozilla.getClipboard();

	// clipboard.setData(deletedFragment.saveXML(), "text/html"); // go back to this once, paste supports html paste!
	clipboard.setData(text, MozClipboard.TEXT_FLAVOR);
	return true;
	
} // end mozileCopy()




// Mozile Cut
// Copy the current selection to the system clipboard, then remove it from the document.
// Works in plain text.

function mozileCut() {
	var f = ["mozileCopyCutPaste.js","mozileCut"];
	mozileDebug(f,2,"Cutting");
	
	mozileDo("Cut");

	var cssr = mozileSelection.getEditableRange();

	if(!cssr || cssr.collapsed) // not an editable area or nothing selected
		return;

	var clipboard = mozilla.getClipboard();

	// data to save - render as text (temporary thing)
	var text = cssr.toString();

	//var deletedFragment = cssr.extractContentsByCSS();

	var objXMLSerializer = new XMLSerializer;
	var deletedFragment = objXMLSerializer.serializeToString(cssr.extractContentsByCSS());

	mozileDebug(f,1,deletedFragment);

	clipboard.setData(deletedFragment, MozClipboard.HTML_FLAVOR); // go back to this once, paste supports html paste!
	clipboard.setData(text, MozClipboard.TEXT_FLAVOR);

	mozileSelection.removeAllRanges();
	mozileSelection.addRange(cssr);
} // end mozileCut()





